searchData={"items":[{"type":"module","title":"Ion","doc":"Lightweight utility library for efficient IO data and chardata handling.","ref":"Ion.html"},{"type":"module","title":"Why `Ion`? - Ion","doc":"See [`README`](./README.md).","ref":"Ion.html#module-why-ion"},{"type":"module","title":"Examples - Ion","doc":"[IO data and chardata](https://hexdocs.pm/elixir/io-and-the-file-system.html#iodata-and-chardata)\ncan be easily be built using:\n\na. interpolation with the [`~i` sigil](`Ion.sigil_i/2`):\n\n      iex> import Ion, only: :sigils\n      iex> ~i\"atom: #{:foo}, number: #{12 + 2.35}\\n\"\n      [\"atom: \", \"foo\", \", number: \", \"14.35\", 10]\n\nb. joining using `join/2` or `map_join/3`:\n\n      iex> Ion.join(100..103, \",\")\n      [[[\"100\", \",\" | \"101\"], \",\" | \"102\"], \",\" | \"103\"]\n\n      iex> Ion.map_join(1..4, \",\", & &1 + 100)\n      [[[\"101\", \",\" | \"102\"], \",\" | \"103\"], \",\" | \"104\"]","ref":"Ion.html#module-examples"},{"type":"function","title":"Ion.iodata_empty?/1","doc":"Returns `true` if `iodata_or_chardata` is empty.\n\nUnlike `IO.iodata_length(iodata) == 0` which needs to walk the complete tree,\nit will bail early as soon as it finds at least one byte or codepoint.\n\nIt should be constant time for most typical outputs, with the exception of\natypical cases where the head is a deep nested tree.\n\nEven if `IO.iodata_length/1` is a very efficient BIF implemented in C, it has a linear\nalgorithmic complexity and can become slow on bigger trees.","ref":"Ion.html#iodata_empty?/1"},{"type":"function","title":"Examples - Ion.iodata_empty?/1","doc":"iex> Ion.iodata_empty?([\"\", []])\n    true\n    iex> Ion.iodata_empty?('a')\n    false\n    iex> Ion.iodata_empty?([\"a\"])\n    false\n    iex> Ion.iodata_empty?([\"\", [], [\"\" | \"c\"]])\n    false","ref":"Ion.html#iodata_empty?/1-examples"},{"type":"function","title":"Ion.join/2","doc":"Joins the given `enumerable` into IO data or chardata using `joiner` as separator.\n\nIt is a fast equivalent to `Enum.join/2`.","ref":"Ion.html#join/2"},{"type":"function","title":"Examples - Ion.join/2","doc":"iex> iodata = Ion.join(1..3)\n    iex> IO.iodata_to_binary(iodata)\n    \"123\"\n\n    iex> iodata = Ion.join(1..3, \" + \")\n    iex> IO.iodata_to_binary(iodata)\n    \"1 + 2 + 3\"","ref":"Ion.html#join/2-examples"},{"type":"function","title":"Ion.map_join/3","doc":"Maps and joins the given `enumerable` as IO data or chardata, in one pass.\n\nIt is a fast equivalent to `Enum.map_join/3`.","ref":"Ion.html#map_join/3"},{"type":"function","title":"Examples - Ion.map_join/3","doc":"iex> iodata = Ion.map_join(1..3, &(&1 * 2))\n    iex> IO.iodata_to_binary(iodata)\n    \"246\"\n\n    iex> iodata = Ion.map_join(1..3, \" + \", &(&1 * 2))\n    iex> IO.iodata_to_binary(iodata)\n    \"2 + 4 + 6\"","ref":"Ion.html#map_join/3-examples"},{"type":"macro","title":"Ion.sigil_i/2","doc":"A sigil to build [IO data](https://hexdocs.pm/elixir/IO.html#module-io-data) or chardata and\navoid string concatenation.\n\nUse `import Ion` to use it, or `import Ion, only: :sigils`.\n\nThis sigil provides a faster version of string interpolation which:\n- will build a list with all chunks instead of concatenating them as a string\n- will keep interpolated lists and binaries untouched, without any validation or transformation\n- will cast anything else using `to_string/1`\n\nWorks with both [IO data](https://hexdocs.pm/elixir/IO.html#module-io-data) and\n[Chardata](https://hexdocs.pm/elixir/IO.html?#module-chardata).\nSee their respective documentation or this\n[guide](https://hexdocs.pm/elixir/io-and-the-file-system.html#iodata-and-chardata)\nfor more information.","ref":"Ion.html#sigil_i/2"},{"type":"macro","title":"Examples - Ion.sigil_i/2","doc":"iex> ~i\"atom: #{:foo}, number: #{12 + 2.35}\\n\"\n    [\"atom: \", \"foo\", \", number: \", \"14.35\", 10]\n\nThe data produced corresponds to the binary that a regular string interpolation\nwould have produced:\n\n    iex> IO.iodata_to_binary(~i\"atom: #{:foo}, number: #{12 + 2.35}\\n\")\n    \"atom: foo, number: 14.35\\n\"\n    iex> \"atom: #{:foo}, number: #{12 + 2.35}\\n\"\n    \"atom: foo, number: 14.35\\n\"\n\nCan be used to build chardata too:\n\n    iex> chardata = ~i\"charlist: #{~c(ジョルノ)}!\"\n    [\"charlist: \", [12472, 12519, 12523, 12494], 33]\n    iex> IO.chardata_to_string(chardata)\n    \"charlist: ジョルノ!\"","ref":"Ion.html#sigil_i/2-examples"},{"type":"extras","title":"Ion","doc":"# Ion\n\n[![Hex Version](https://img.shields.io/hexpm/v/ion.svg)](https://hex.pm/packages/ion)\n[![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/ion/)\n[![CI](https://github.com/sabiwara/ion/workflows/CI/badge.svg)](https://github.com/sabiwara/ion/actions?query=workflow%3ACI)\n\nLightweight utility library for efficient IO data and chardata handling.","ref":"readme.html"},{"type":"extras","title":"TLDR - Ion","doc":"Interpolation:\n\n```elixir\n# plain string\n\"#{name}: #{count}.\"\n# IO data (using Ion)\n~i\"#{name}: #{count}.\"\n# IO data (manual)\n[name, \" \", to_string(count), ?.]\n```\n\nJoining:\n\n```elixir\n# plain string\nEnum.join(integers, \"+\")\n# IO data (using Ion)\nIon.join(integers, \"+\")\n# IO data (manual)\nEnum.map_intersperse(integers, \"+\", &to_string/1)\n```\n\nMap joining:\n\n```elixir\n# plain string\nEnum.map_join(users, \", \", fn user -> \"#{user.first}.#{user.last}@passione.org\" end)\n# IO data (using Ion)\nIon.map_join(users, \", \", fn user -> ~i\"#{user.first}.#{user.last}@passione.org\" end)\n# IO data (manual)\nEnum.map_intersperse(users, \", \", fn user -> [user.first, ?., user.last, \"@passione.org\"] end)\n```","ref":"readme.html#tldr"},{"type":"extras","title":"Why `Ion`? - Ion","doc":"[IO data and chardata](https://hexdocs.pm/elixir/io-and-the-file-system.html#iodata-and-chardata)\nare one of the secret weapons behind Elixir and Erlang performance when it comes\nto string processing and IO. Turns out the fastest way to concatenate strings\nis: avoiding concatenation in the first place!\n\nWhile it is perfectly possible to handcraft IO data with just the standard\nlibrary, it can sometimes be tedious, cryptic and error prone. `Ion` provides a\nfew common recipes which:\n\n- drop-in replacement, with APIS consistent with the standard way of building\n  strings\n- reduce the cognitive overhead and make the intent explicit\n- are implemented in an optimal fashion (`Ion` is fast!)\n- help reducing bugs through better typing (see below)\n\nThe examples above illustrate how easy it is to migrate from building strings to\nbuilding IO data or chardata.","ref":"readme.html#why-ion"},{"type":"extras","title":"Increased safety - Ion","doc":"Building IO lists manually or through interspersing is error prone: we need to\nbe careful to cast things that are neither strings nor nested IO data or will\nend up with invalid data:\n\n```elixir\niex> as_bytes = Enum.intersperse(100..105, \"+\")\n[100, \"+\", 101, \"+\", 102, \"+\", 103, \"+\", 104, \"+\", 105]\niex> IO.iodata_to_binary(as_bytes)\n\"d+e+f+g+h+i\"\n# need to make sure we have strings:\niex> Enum.map_intersperse(100..105, \"+\", &to_string/1) |> IO.iodata_to_binary()\n# works just like Enum.join/2:\niex> Ion.join(100..105, \"+\") |> IO.iodata_to_binary()\n\"100+101+102+103+104+105\"\n```","ref":"readme.html#increased-safety"},{"type":"extras","title":"Performance - Ion","doc":"Because they are specialized, the join functions should also be faster than\ninterspersing (~1.5x, see the `benchmarks` folder).","ref":"readme.html#performance"},{"type":"extras","title":"Installation - Ion","doc":"Ion can be installed by adding `Ion` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ion, \"~> 0.1.0\"}\n  ]\nend\n```\n\nOr you can just try it out from `iex` or an `.exs` script:\n\n```elixir\niex> Mix.install([:ion])\n:ok\niex> Ion.join([\"Hello\", :world], \",\")\n[\"Hello\", \",\", \"world\"]\n```\n\nDocumentation can be found at [https://hexdocs.pm/ion](https://hexdocs.pm/ion).","ref":"readme.html#installation"},{"type":"extras","title":"Copyright and License - Ion","doc":"Ion is licensed under the [MIT License](LICENSE.md).","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Dev - Changelog","doc":"","ref":"changelog.html#dev"},{"type":"extras","title":"v0.1.0 (2024-04-25) - Changelog","doc":"- Initial release","ref":"changelog.html#v0-1-0-2024-04-25"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2024 Sabiwara\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the \"Software\"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of\nthe Software, and to permit persons to whom the Software is furnished to do so,\nsubject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS\nFOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\nCOPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER\nIN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN\nCONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}